@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -36,13 +36,18 @@
 #include "gmime-part.h"
 #include "gmime-error.h"
 
-
+#ifdef ENABLE_DEBUG
+#define d(x) x
+#else
 #define d(x)
+#endif
+
+#define _(x) x
 
 
 /**
  * SECTION: gmime-multipart-encrypted
- * @title: GMimeMultpartEncrypted
+ * @title: GMimeMultipartEncrypted
  * @short_description: Encrypted MIME multiparts
  * @see_also: #GMimeMultipart
  *
@@ -99,21 +104,12 @@ g_mime_multipart_encrypted_class_init (GMimeMultipartEncryptedClass *klass)
 static void
 g_mime_multipart_encrypted_init (GMimeMultipartEncrypted *mpe, GMimeMultipartEncryptedClass *klass)
 {
-	mpe->decrypted = NULL;
-	mpe->validity = NULL;
+	
 }
 
 static void
 g_mime_multipart_encrypted_finalize (GObject *object)
 {
-	GMimeMultipartEncrypted *mpe = (GMimeMultipartEncrypted *) object;
-	
-	if (mpe->decrypted)
-		g_object_unref (mpe->decrypted);
-	
-	if (mpe->validity)
-		g_mime_signature_validity_free (mpe->validity);
-	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -145,9 +141,10 @@ g_mime_multipart_encrypted_new (void)
  * g_mime_multipart_encrypted_encrypt:
  * @mpe: multipart/encrypted object
  * @content: MIME part to encrypt
- * @ctx: encryption cipher context
+ * @ctx: encryption context
  * @sign: %TRUE if the content should also be signed or %FALSE otherwise
  * @userid: user id to use for signing (only used if @sign is %TRUE)
+ * @hash: digest algorithm to use when signing
  * @recipients: an array of recipients to encrypt to
  * @err: a #GError
  *
@@ -162,21 +159,26 @@ g_mime_multipart_encrypted_new (void)
  **/
 int
 g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *content,
-				    GMimeCipherContext *ctx, gboolean sign,
-				    const char *userid, GPtrArray *recipients,
-				    GError **err)
+				    GMimeCryptoContext *ctx, gboolean sign,
+				    const char *userid, GMimeCryptoHash hash,
+				    GPtrArray *recipients, GError **err)
 {
 	GMimeStream *filtered_stream, *ciphertext, *stream;
 	GMimePart *version_part, *encrypted_part;
 	GMimeContentType *content_type;
 	GMimeDataWrapper *wrapper;
 	GMimeFilter *crlf_filter;
+	const char *protocol;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), -1);
-	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), -1);
-	g_return_val_if_fail (ctx->encrypt_protocol != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
 	g_return_val_if_fail (GMIME_IS_OBJECT (content), -1);
 	
+	if (!(protocol = g_mime_crypto_context_get_encryption_protocol (ctx))) {
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("Encryption not supported."));
+		return -1;
+	}
+	
 	/* get the cleartext */
 	stream = g_mime_stream_mem_new ();
 	filtered_stream = g_mime_stream_filter_new (stream);
@@ -194,7 +196,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	
 	/* encrypt the content stream */
 	ciphertext = g_mime_stream_mem_new ();
-	if (g_mime_cipher_context_encrypt (ctx, sign, userid, recipients, stream, ciphertext, err) == -1) {
+	if (g_mime_crypto_context_encrypt (ctx, sign, userid, hash, recipients, stream, ciphertext, err) == -1) {
 		g_object_unref (ciphertext);
 		g_object_unref (stream);
 		return -1;
@@ -204,11 +206,11 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	g_mime_stream_reset (ciphertext);
 	
 	/* construct the version part */
-	content_type = g_mime_content_type_new_from_string (ctx->encrypt_protocol);
+	content_type = g_mime_content_type_new_from_string (protocol);
 	version_part = g_mime_part_new_with_type (content_type->type, content_type->subtype);
 	g_object_unref (content_type);
 	
-	content_type = g_mime_content_type_new_from_string (ctx->encrypt_protocol);
+	content_type = g_mime_content_type_new_from_string (protocol);
 	g_mime_object_set_content_type (GMIME_OBJECT (version_part), content_type);
 	g_mime_part_set_content_encoding (version_part, GMIME_CONTENT_ENCODING_7BIT);
 	stream = g_mime_stream_mem_new_with_buffer ("Version: 1\n", strlen ("Version: 1\n"));
@@ -217,9 +219,6 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	g_object_unref (wrapper);
 	g_object_unref (stream);
 	
-	mpe->decrypted = content;
-	g_object_ref (content);
-	
 	/* construct the encrypted mime part */
 	encrypted_part = g_mime_part_new_with_type ("application", "octet-stream");
 	g_mime_part_set_content_encoding (encrypted_part, GMIME_CONTENT_ENCODING_7BIT);
@@ -236,8 +235,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	g_object_unref (version_part);
 	
 	/* set the content-type params for this multipart/encrypted part */
-	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mpe), "protocol",
-						  ctx->encrypt_protocol);
+	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mpe), "protocol", protocol);
 	g_mime_multipart_set_boundary (GMIME_MULTIPART (mpe), NULL);
 	
 	return 0;
@@ -247,57 +245,63 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 /**
  * g_mime_multipart_encrypted_decrypt:
  * @mpe: multipart/encrypted object
- * @ctx: decryption cipher context
+ * @ctx: decryption context
+ * @result: a #GMimeDecryptionResult
  * @err: a #GError
  *
  * Attempts to decrypt the encrypted MIME part contained within the
  * multipart/encrypted object @mpe using the @ctx decryption context.
  *
- * If @validity is non-NULL, then on a successful decrypt operation,
- * it will be updated to point to a newly-allocated
- * #GMimeSignatureValidity with signature status information.
+ * If @result is non-NULL, then on a successful decrypt operation, it will be
+ * updated to point to a newly-allocated #GMimeDecryptionResult with signature
+ * status information as well as a list of recipients that the part was
+ * encrypted to.
  *
  * Returns: the decrypted MIME part on success or %NULL on fail. If the
  * decryption fails, an exception will be set on @err to provide
  * information as to why the failure occured.
  **/
 GMimeObject *
-g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherContext *ctx,
-				    GError **err)
+g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoContext *ctx,
+				    GMimeDecryptionResult **result, GError **err)
 {
 	GMimeObject *decrypted, *version, *encrypted;
 	GMimeStream *stream, *ciphertext;
+	const char *protocol, *supported;
 	GMimeStream *filtered_stream;
 	GMimeContentType *mime_type;
-	GMimeSignatureValidity *sv;
+	GMimeDecryptionResult *res;
 	GMimeDataWrapper *wrapper;
 	GMimeFilter *crlf_filter;
 	GMimeParser *parser;
-	const char *protocol;
 	char *content_type;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), NULL);
-	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
-	g_return_val_if_fail (ctx->encrypt_protocol != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
 	
-	if (mpe->decrypted) {
-		/* we seem to have already decrypted the part */
-		return mpe->decrypted;
-	}
+	if (result)
+		*result = NULL;
 	
 	protocol = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mpe), "protocol");
+	supported = g_mime_crypto_context_get_encryption_protocol (ctx);
 	
 	if (protocol) {
-		/* make sure the protocol matches the cipher encrypt protocol */
-		if (g_ascii_strcasecmp (ctx->encrypt_protocol, protocol) != 0) {
+		/* make sure the protocol matches the crypto encrypt protocol */
+		if (!supported || g_ascii_strcasecmp (supported, protocol) != 0) {
 			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
-				     "Failed to decrypt MIME part: protocol error");
+				     _("Cannot decrypt multipart/encrypted part: unsupported encryption protocol '%s'."),
+				     protocol);
 			
 			return NULL;
 		}
-	} else {
+	} else if (supported != NULL) {
 		/* *shrug* - I guess just go on as if they match? */
-		protocol = ctx->encrypt_protocol;
+		protocol = supported;
+	} else {
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+				     _("Cannot decrypt multipart/encrypted part: unspecified encryption protocol."));
+		
+		return NULL;
 	}
 	
 	version = g_mime_multipart_get_part (GMIME_MULTIPART (mpe), GMIME_MULTIPART_ENCRYPTED_VERSION);
@@ -305,8 +309,8 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	/* make sure the protocol matches the version part's content-type */
 	content_type = g_mime_content_type_to_string (version->content_type);
 	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
-		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
-			     "Failed to decrypt MIME part: protocol error");
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Cannot decrypt multipart/encrypted part: content-type does not match protocol."));
 		
 		g_free (content_type);
 		
@@ -318,7 +322,9 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	encrypted = g_mime_multipart_get_part (GMIME_MULTIPART (mpe), GMIME_MULTIPART_ENCRYPTED_CONTENT);
 	mime_type = g_mime_object_get_content_type (encrypted);
 	if (!g_mime_content_type_is_type (mime_type, "application", "octet-stream")) {
-		g_object_unref (encrypted);
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Cannot decrypt multipart/encrypted part: unexpected content type."));
+		
 		return NULL;
 	}
 	
@@ -334,7 +340,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	g_object_unref (crlf_filter);
 	
 	/* get the cleartext */
-	if (!(sv = g_mime_cipher_context_decrypt (ctx, ciphertext, filtered_stream, err))) {
+	if (!(res = g_mime_crypto_context_decrypt (ctx, ciphertext, filtered_stream, err))) {
 		g_object_unref (filtered_stream);
 		g_object_unref (stream);
 		
@@ -353,37 +359,16 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	g_object_unref (parser);
 	
 	if (!decrypted) {
-		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
-			     "Failed to decrypt MIME part: parse error");
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Cannot decrypt multipart/encrypted part: failed to parse decrypted content."));
 		
-		g_mime_signature_validity_free (sv);
+		g_mime_decryption_result_free (res);
 		
 		return NULL;
 	}
 	
-	/* cache the decrypted part */
-	mpe->decrypted = decrypted;
-	mpe->validity = sv;
+	if (result)
+		*result = res;
 	
 	return decrypted;
 }
-
-
-/**
- * g_mime_multipart_encrypted_get_signature_validity:
- * @mpe: a #GMimeMultipartEncrypted
- *
- * Gets the signature validity of the encrypted MIME part.
- *
- * Note: This is only useful after calling
- * g_mime_multipart_encrypted_decrypt().
- *
- * Returns: a #GMimeSignatureValidity.
- **/
-const GMimeSignatureValidity *
-g_mime_multipart_encrypted_get_signature_validity (GMimeMultipartEncrypted *mpe)
-{
-	g_return_val_if_fail (GMIME_IS_MULTIPART_ENCRYPTED (mpe), NULL);
-	
-	return mpe->validity;
-}
